"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts_packet_1 = require("./ts-packet");
var ts_pes_1 = require("./ts-pes");
var is_complete_pes = function (pes) {
    return (ts_packet_1.HEADER_SIZE + ts_pes_1.PES_packet_length(pes) - pes.length) <= 0;
};
var SectionQueue = /** @class */ (function () {
    function SectionQueue() {
        this.queue = [];
        this.partialPES = Buffer.from([]);
    }
    SectionQueue.prototype.push = function (packet) {
        var begin = ts_packet_1.HEADER_SIZE + (ts_packet_1.has_adaptation_field(packet) ? 1 + ts_packet_1.adaptation_field_length(packet) : 0);
        if (this.partialPES.length == 0 && !ts_packet_1.payload_unit_start_indicator(packet)) {
            return;
        }
        if (ts_packet_1.payload_unit_start_indicator(packet)) {
            var next = begin + Math.max(0, ts_pes_1.PES_HEADER_SIZE + ts_pes_1.PES_packet_length(packet.slice(begin)));
            this.partialPES = packet.slice(begin, next);
        }
        else {
            var next = begin + Math.max(0, ts_pes_1.PES_HEADER_SIZE + ts_pes_1.PES_packet_length(this.partialPES) - this.partialPES.length);
            this.partialPES = Buffer.concat([this.partialPES, packet.slice(begin, next)]);
        }
        if (is_complete_pes(this.partialPES)) {
            this.queue.push(this.partialPES);
            this.partialPES = Buffer.from([]);
        }
    };
    SectionQueue.prototype.pop = function () {
        return this.queue.shift();
    };
    SectionQueue.prototype.isEmpty = function () {
        return this.queue.length === 0;
    };
    SectionQueue.prototype.clear = function () {
        this.partialPES = Buffer.from([]);
        this.queue.length = 0;
    };
    return SectionQueue;
}());
exports.default = SectionQueue;
