"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts_packet_1 = require("./ts-packet");
var ts_section_1 = require("./ts-section");
var is_complete_section = function (section) {
    return (ts_section_1.BASIC_HEADER_SIZE + ts_section_1.section_length(section) - section.length) <= 0;
};
var SectionQueue = /** @class */ (function () {
    function SectionQueue() {
        this.queue = [];
        this.partialSection = Buffer.from([]);
    }
    SectionQueue.prototype.push = function (packet) {
        var begin = ts_packet_1.HEADER_SIZE + (ts_packet_1.has_adaptation_field(packet) ? 1 + ts_packet_1.adaptation_field_length(packet) : 0);
        if (ts_packet_1.payload_unit_start_indicator(packet)) {
            begin += 1;
        }
        if (this.partialSection.length == 0) {
            if (ts_packet_1.payload_unit_start_indicator(packet)) {
                begin += ts_packet_1.pointer_field(packet);
            }
            else {
                return;
            }
        }
        if (ts_packet_1.payload_unit_start_indicator(packet)) {
            while (begin < ts_packet_1.PACKET_SIZE) {
                if (packet[begin] === ts_packet_1.STUFFING_BYTE) {
                    break;
                }
                if (this.partialSection.length == 0) {
                    var next = begin + Math.max(0, ts_section_1.BASIC_HEADER_SIZE + ts_section_1.section_length(packet.slice(begin)));
                    this.partialSection = Buffer.concat([this.partialSection, packet.slice(begin, next)]);
                    begin = next;
                }
                else {
                    var next = begin + Math.max(0, ts_section_1.BASIC_HEADER_SIZE + ts_section_1.section_length(this.partialSection) - this.partialSection.length);
                    this.partialSection = Buffer.concat([this.partialSection, packet.slice(begin, next)]);
                    begin = next;
                }
                if (is_complete_section(this.partialSection)) {
                    this.queue.push(this.partialSection);
                    this.partialSection = Buffer.from([]);
                }
            }
        }
        else {
            var next = begin + Math.max(0, ts_section_1.BASIC_HEADER_SIZE + ts_section_1.section_length(this.partialSection) - this.partialSection.length);
            this.partialSection = Buffer.concat([this.partialSection, packet.slice(begin, next)]);
            if (is_complete_section(this.partialSection)) {
                this.queue.push(this.partialSection);
                this.partialSection = Buffer.from([]);
            }
        }
    };
    SectionQueue.prototype.pop = function () {
        return this.queue.shift();
    };
    SectionQueue.prototype.isEmpty = function () {
        return this.queue.length === 0;
    };
    SectionQueue.prototype.clear = function () {
        this.partialSection = Buffer.from([]);
        this.queue.length = 0;
    };
    return SectionQueue;
}());
exports.default = SectionQueue;
